/*
 * Project name:
      DAC for VCF Function Generator (Driving external Digital-to-Analog converter)
 * Copyright:
     (c) MikroElektronika, 2012
 * Revision History:
     20080930:
       - initial release;
     20111011;
       - revision (JK);
     20140218;
       - revision (Philippe LE GUEN - http://pleguen.fr/ )
 * Description:
     This is a program which use the Microchip's
     MCP4921 12-bit D/A converter with PIC MCUs. This device accepts digital
     input (number from 0..4095) and transforms it to the output voltage,
     ranging from 0..Vref. In this case, the DAC communicates with MCU
     through the SPI communication.
     Buttons RA1 are used to change Delay value (Slow down key repeat pace)
     Buttons RA3 and RA4 are used to change value sent to DAC.

 * Configuration:
     MCU:             PIC18F25K22/02
     Dev.Board:       Ready for PIC
     Oscillator:      HS, 8 MHz (HS-PLL, 32.00000 MHz)
     Ext. Modules:    MPC 4921 on PORTC, LCD adaptator 2x16 on PORTB
     SW:              mikroC PRO v6.0.1 for PIC
                      http://www.mikroe.com/mikroc/pic/
 * NOTES:
     - Turn on PIC and CS0 switches on DAC extra board.
     - Put button jumper (J17) into VCC position and pull-down PORTA (PORTA three-state switch). (board specific)
     - Turn on LCD/GLCD backlight switch SW4.6.
     - The value of the DAC is stored into EEPROM of C.
     - RA1 are input key for preset DAC_delay (1..1000ms) by 6 steps
     - RA3..RA5 are input keys for Increment, Decrement and reset DAC_value
     - RA0 are Input, and used to select the current duty for PWM1 (LDR).
     - RC2 are Output, and used by PWM1 for select the LCD luminosity.
     - Pull-down RA0..RA1, RA3..RA5, RC2.
     
 * mikroBootloader:
     - COM Port: COM8
     - Baud Rate: 115200bps
 */

#include "Init_LCD adaptator 2x16_P18F45K22_PortB.h"

// DAC module connections
sbit Chip_Select at LATC1_bit;
sbit Chip_Select_Direction at TRISC1_bit;
// End DAC module connections

unsigned int value, adc_rd, adc_ldr, data_, temp, High_Data, Low_Data, value_temp, cnt, speed_delay;
unsigned char ch, val;
char *text;
long tlong;
unsigned long k;
char ii;                                             // loop variable
unsigned short current_duty;

#define PUSH_DAC_Delay      (RA1_bit == 1)
#define PUSH_Increment      (RA3_bit == 1)
#define PUSH_Decrement      (RA4_bit == 1)
#define PUSH_Preset_mini    (RA5_bit == 1)
#define PUSH_Preset_maxi    (RA3_bit && RA4_bit == 1)

//-------------- Input / Output -------------
/*
RA0 : Input ADC (AN0) for LDR_value, and used to select the current duty for PWM1
RA1 : Input key for preset DAC_delay (1..1000ms)
RA3 : Input key for Increment DAC_value
RA4 : Input key for Decrement DAC_value
RA5 : Input key for Reset DAC_value
RA3 + RA4 : input keys for preset DAC_value to 5000mV

RC2 : Output used by PWM1 for select the LCD luminosity (LDR)
*/

void delay2S(){                                      // 2 seconds delay function
  Delay_ms(2000);
}

void InitMain() {
  TRISA0_bit = 1;                                    // Set RA0 pin as input (for LDR detection)
  TRISA1_bit = 1;                                    // Set RA1 pin as input (key for DAC_delay)
  TRISA3_bit = 1;                                    // Set RA3 pin as input (key for increment)
  TRISA4_bit = 1;                                    // Set RA4 pin as input (key for decrement)
  TRISA5_bit = 1;                                    // Set RA5 pin as input (key for reset value to 0)
  Chip_Select = 1;                                   // Deselect DAC
  Chip_Select_Direction = 0;                         // Set CS# pin as Output
  SPI1_Init();                                       // Initialize SPI module
  PWM1_Init(5000);                                   // Initialize PWM1 module at 5KHz
                                                     // Sets PWM duty ratio. Parameter duty takes values from 0 to 255,
                                                     //   where 0 is 0%, 127 is 50%, and 255 is 100% duty ratio. 
                                                     //   Other specific values for duty ratio can be calculated
                                                     //   as (Percent*255)/100.

  Lcd_Init();                                        // LCD display initialization
  Lcd_Cmd(_LCD_CURSOR_OFF);                          // LCD command (cursor off)
  Lcd_Cmd(_LCD_CLEAR);                               // LCD command (clear LCD)
  text = "Initializing...";                          // Define the first message
  Lcd_Out(1,1,text);
  Delay_ms(500);
  Lcd_Cmd(_LCD_CLEAR);
  Lcd_Out(1,1,"VCF Generator");
  text = "DAC on LCD";                               // Define the first message
  Lcd_Out(2,2,text);                                 // Write the first message in the first line
  Delay_ms(1000);
  Lcd_Cmd(_LCD_CLEAR);
  Lcd_Out(1,1,"Reading EEPROM");
  Lcd_Out(2,1,"from PIC...");
  Delay_ms(1000);
  Lcd_Cmd(_LCD_CLEAR);
  text = "Output Voltage";                           // Define the second message
  Lcd_Out(1,1,text);                                 // Define the first message
  Delay_ms(250);
  text = "for VCF:";                                 // Define the third message
}

void Write_EEPROM(unsigned int data_) {
  // Send High Byte
  High_Data = (data_ >> 8) & 0x0F;                  // Write value[11..8] to High_Data[3..0] at address 8 (poids fort)
  EEPROM_Write(0x08,High_Data);
  
  // Send Low Byte
  Low_Data = data_;                                 // Write value[7..0] to Low_Data[7..0] at address 9 (poids faible)
  EEPROM_Write(0x09,Low_Data);
}

// DAC increments (0..4095) --> output voltage (0..Vref)
void DAC_Output(unsigned int valueDAC) {
  Chip_Select = 0;                                   // Select DAC chip

  // Send High Byte
  temp = (valueDAC >> 8) & 0x0F;                     // Store valueDAC[11..8] to temp[3..0]
  temp |= 0x30;                                      // Define DAC setting, see MCP4921 datasheet
  SPI1_Write(temp);                                  // Send high byte via SPI

  // Send Low Byte
  temp = valueDAC;                                   // Store valueDAC[7..0] to temp[7..0]
  SPI1_Write(temp);                                  // Send low byte via SPI

  Chip_Select = 1;                                   // Deselect DAC chip
}

void Display_speed_delay() {
  val = ((speed_delay / 1000) % 10) ;              // Extract thousands
  Lcd_Chr(1,12,48+val);                            // Write result in ASCII format
  val = ((speed_delay / 100) % 10) ;               // Extract hundreds
  Lcd_Chr_CP(48+val);                              // Write result in ASCII format
  val = ((speed_delay / 10) % 10) ;                // Extract tens of millivolts
  Lcd_Chr_CP(48+val);                              // Write result in ASCII format
  val = (speed_delay % 10) ;                       // Extract units
  Lcd_Chr_CP(48+val);                              // Write result in ASCII format

  Lcd_Out(1,1,"Delay(ms): ");
  Delay_ms(2000);
  Lcd_Out(1,1,"Output Voltage  ");
}

void main() {
  ANSELA = 0;                                        // Configure PORTA pins as digital
  //ANSELB = 0x01;                                     // Configure PORTB pins as digital
                                                     //  and configure RB0 as analog
  ANSELC = 0;                                        // Configure PORTC pins as digital
  SLRCON = 0;                                        // Set output slew rate on all ports at standard rate

  InitMain();                                        // Perform main initialization
  current_duty  = 16;                                // initial value for current_duty
  PWM1_Start();                                      // start PWM1 (RC1)
  PWM1_Set_Duty(current_duty);                       // Set current duty for PWM1

  cnt == 4;                                           // Preset value of cnt for DAC_delay
  
  // Reading High Byte
  High_Data = EEPROM_Read(0x08);                     // Read value from High_Data[7..0] at address 8 (MSB)
  
  // Reading Low Byte
  Low_Data = EEPROM_Read(0x09);                      // Read value from Low_Data[7..0] at address 9 (LSB)
  value = (High_Data << 8) | (unsigned int)Low_Data; // Moves the 8-bits to the left, and make a bitwise (inclusive) OR
                                                     //   the result (16-bits) are store to "value"


 while (1) {                                         // Endless loop
 
    adc_ldr = ADC_Read(0);                           // get ADC value from channel AN0 (RA0)
    if (adc_ldr < 3) {                               // Fix the minimum value of LDR detection
      adc_ldr = 3;
    }
    if ((PUSH_Increment) && (value < 4095)) {        // If RA3 button is pressed
      value++;                                       //   increment value
      Write_EEPROM(value);
    }
    else {
      if ((PUSH_Decrement) && (value > 0)) {         // If RA4 button is pressed
        value--;                                     //   decrement value
        Write_EEPROM(value);
      }
    }
    if (PUSH_Preset_maxi == 1) {                     // If RA3 and RA4 button is pressed
      value = 4095;                                  //   set value to 4095
      Write_EEPROM(value);
    }
    if (PUSH_Preset_mini == 1) {                     // If RA5 buton is pressed
      value = 0;                                     //   reset value to 0
      Write_EEPROM(value);
    }
    
    DAC_Output(value);                               // Send value to DAC chip

    if (PUSH_DAC_Delay) {
      Display_speed_delay();
      ++cnt;
      //Display_speed_delay();
      while(PUSH_DAC_Delay);                         // attendre que le bouton soit relach
      //Display_speed_delay();
    }
    if (cnt > 5) {
      cnt = 0;
      Delay_ms(1);
      speed_delay = 1;
    }
    else
    if (cnt == 0) {
      Delay_ms(1);
      speed_delay = 1;
    }
    if (cnt == 1) {
      Delay_ms(10);
      speed_delay = 10;
    }
    if (cnt == 2) {
      Delay_ms(50);
      speed_delay = 50;
    }
    if (cnt == 3) {
      Delay_ms(250);
      speed_delay = 250;
    }
    if (cnt == 4) {
      Delay_ms(500);
      speed_delay = 500;
    }
    if (cnt == 5) {
      Delay_ms(1000);
      speed_delay = 1000;
    }

/*if (PUSH_DAC_Delay) {
      Display_speed_delay();
      //Delay_ms(100);                               // dlai qui ne sert pas  grand chose, juste aux tests...
      ++cnt;
      //Display_speed_delay();
      while(PUSH_DAC_Delay);                         // attendre que le bouton soit relach
      //Display_speed_delay();
    }*/
    

    current_duty = adc_ldr * 1.5;                    // Coefficient for adjust LDR level detection
    PWM1_Set_Duty(current_duty);


                                       //----- Display on LCD -----
    adc_rd = value;
    Lcd_Out(2,1,text);
    tlong = (long)adc_rd * 5000;                     // Convert the result in millivolts
    //tlong = tlong / 1023;                            // 0..1023 -> 0-5000mV (10-bit)
    k = 1.221001221;
    tlong = (tlong / 4095) * k;                      // 0..4095 -> 0-5000mV (12-bit) --> 1,221001221E0 by step
    ch = tlong / 1000;                               // Extract volts (thousands of millivolts)
                                                     // from result
    ch = ((tlong / 1000) % 10) ;                     // Extract thousands of millivolts
    Lcd_Chr(2,10,48+ch);
    ch = ((tlong / 100) % 10) ;                      // Extract hundreds of millivolts
    Lcd_Chr_CP(48+ch);                               // Write result in ASCII format
    ch = ((tlong / 10) % 10) ;                       // Extract tens of millivolts
    Lcd_Chr_CP(48+ch);                               // Write result in ASCII format
    ch = (tlong % 10) ;                              // Extract digits for millivolts
    Lcd_Chr_CP(48+ch);                               // Write result in ASCII format
    Lcd_Out_CP("mV");
  }
}